////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : DevExpress VCL Support
//  * Unit Name : FWBackStageButton
//  * Purpose   :    BackStage 
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2013.
//  * Version   : 1.0.3
//  * Home Page : http://rouse.drkb.ru
//  * Home Blog : http://alexander-bagel.blogspot.ru
//  ****************************************************************************
//

// ShowCorner  Description    IconPosition = ipLeft
//  ImageList- .   Images < LargeImages < TileImages
//     .

unit FWBackStageButton;

interface

uses
  Windows,
  Messages,
  Classes,
  Controls,
  Themes,
  Graphics,
  SysUtils,
  ImgList,
  ActnList,
  dxRibbon,
  dxBarSkinConsts,
  dxGDIPlusClasses,
  dxGDIPlusAPI;

const
  CM_THEMECHANGED = CM_BASE + WM_THEMECHANGED;

type
  EBackStageButtonException = class(Exception);
  TIconPosition = (ipLeft, ipTop);

  TDropDownEvent = procedure(Sender: TObject; X, Y, AWidth: Integer) of object;

  TFWBackStageButton = class(TGraphicControl)
  private
    FRibbon: TdxRibbon;
    FIcoRect, FCaptionRect, FDescriptionRect: TRect;
    FCaptionSingleLine, FDescriptionSingleLine: Boolean;
    FDropDownLeftOffset: Integer;
    FTileImages, FLargeImages, FSmallImages: TCustomImageList;
    FTileChangeLink, FLargeChangeLink, FSmallChangeLink: TChangeLink;
    FTileImageIndex, FLargeImageIndex, FSmallImageIndex: Integer;
    FButtonState: Integer;
    FCaptionColor, FDescriptionColor: TColor;
    FShowDropDownMark: Boolean;
    FShowCorner: Boolean;
    FShowBorder: Boolean;
    FShowPin: Boolean;
    FIconPosition: TIconPosition;
    FSubStrings: string;
    FLocked: Boolean;
    FUsedImageListIndex: Integer;
    FBackGround: TBitmap;
    FDown: Boolean;
    FDropDown: TDropDownEvent;
    FAutoCheck: Boolean;
    FAlignment: TAlignment;
    FTransparentSet: Boolean;
    procedure SetRibbon(const Value: TdxRibbon);
    procedure SetShowDropDownMark(const Value: Boolean);
    procedure SetIconPosition(const Value: TIconPosition);
    procedure SetShowBorder(const Value: Boolean);
    procedure SetShowCorner(const Value: Boolean);
    procedure SetSubStrings(const Value: string);
    procedure SetImageIndex(const Index, Value: Integer);
    procedure SetImages(const Index: Integer; const Value: TCustomImageList);
    procedure SetShowPin(const Value: Boolean);
    procedure SetDown(const Value: Boolean);
    procedure SetCaptionColor(const Value: TColor);
    procedure SetDescriptionColor(const Value: TColor);
    procedure SetAlignment(const Value: TAlignment);
    function GetTransparent: Boolean;
    procedure SetTransparent(const Value: Boolean);
  protected
    function GetActionLinkClass: TControlActionLinkClass; override;
    function IsAutoCheckStored: Boolean;
    function IsDownStored: Boolean;
    function IsImageIndexStored(const Index: Integer): Boolean;
    function IsParentVisible: Boolean;
    procedure CalcButtonStateAftefEvent;
  protected
    procedure Paint; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMThemeChanged(var Message: TMessage); message CM_THEMECHANGED;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
    procedure ImageListChange(Sender: TObject);
    procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
  protected
    procedure CalcItemRects;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure Lock;
    procedure Unlock;
  published
    property Action;
    property Align;
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property AutoCheck: Boolean read FAutoCheck write FAutoCheck stored IsAutoCheckStored default False;
    property AutoSize;
    property Caption;
    property CaptionColor: TColor read FCaptionColor write SetCaptionColor default $262626;
    property Color;
    property Constraints;
    property Description: string read FSubStrings write SetSubStrings;
    property DescriptionColor: TColor read FDescriptionColor write SetDescriptionColor default clGrayText;
    property Down: Boolean read FDown write SetDown stored IsDownStored default False;
    property Enabled;
    property ImageIndex: Integer index 0 read FSmallImageIndex write SetImageIndex stored IsImageIndexStored default -1;
    property Images: TCustomImageList index 0 read FSmallImages write SetImages;
    property IconPosition: TIconPosition read FIconPosition write SetIconPosition default ipLeft;
    property Hint;
    property LargeImageIndex: Integer index 1 read FLargeImageIndex write SetImageIndex default -1;
    property LargeImageList: TCustomImageList index 1 read FLargeImages write SetImages;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property Ribbon: TdxRibbon read FRibbon write SetRibbon;
    property ShowBorder: Boolean read FShowBorder write SetShowBorder default True;
    property ShowCorner: Boolean read FShowCorner write SetShowCorner default False;
    property ShowDropDownMark: Boolean read FShowDropDownMark write SetShowDropDownMark default False;
    property ShowHint;
    property ShowPin: Boolean read FShowPin write SetShowPin default False;
    property TileImageIndex: Integer index 2 read FTileImageIndex write SetImageIndex default -1;
    property TileImageList: TCustomImageList index 2 read FTileImages write SetImages;
    property Transparent: Boolean read GetTransparent write SetTransparent stored FTransparentSet;
    property Visible;
    property OnClick;
    property OnDropDown: TDropDownEvent read FDropDown write FDropDown;
  end;

  TFWBackStageButtonActionLink = class(TControlActionLink)
  protected
    FClient: TFWBackStageButton;
    procedure AssignClient(AClient: TObject); override;
    function  IsCheckedLinked: Boolean; override;
    procedure SetChecked(Value: Boolean); override;
    function  IsImageIndexLinked: Boolean; override;
    procedure SetImageIndex(Value: Integer); override;
  end;

  procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Fangorn Wizards Lab', [TFWBackStageButton]);
end;

{ TFWBackStageButton }

procedure TFWBackStageButton.ActionChange(Sender: TObject; CheckDefaults: Boolean);
begin
  inherited;
  if Action is TCustomAction then
    with TCustomAction(Sender) do
    begin
      if not CheckDefaults or (Self.Caption = '') then
        Self.Caption := Caption;
      if not CheckDefaults then
        Self.AutoCheck := AutoCheck;
      if (Self is TFWBackStageButton) and
        (not CheckDefaults or (TFWBackStageButton(Self).Down = False)) then
        TFWBackStageButton(Self).Down := Checked;
      if not CheckDefaults or (Self.Enabled = True) then
        Self.Enabled := Enabled;
      if not CheckDefaults or (Self.Hint = '') then
        Self.Hint := Hint;
      if not CheckDefaults or (Self.ImageIndex = -1) then
        Self.ImageIndex := ImageIndex;
      if not CheckDefaults or (Self.Visible = True) then
        Self.Visible := Visible;
      if not CheckDefaults or not Assigned(Self.OnClick) then
        Self.OnClick := OnExecute;
    end;
end;

procedure TFWBackStageButton.CalcButtonStateAftefEvent;
var
  P: TPoint;
begin
  if IsParentVisible then
  begin
    GetCursorPos(P);
    if not PtInRect(ClientRect, ScreenToClient(P)) then
      FButtonState := DXBAR_NORMAL
    else
      FButtonState := DXBAR_HOT;
  end
  else
    FButtonState := DXBAR_NORMAL;
  Invalidate;
end;

procedure TFWBackStageButton.CalcItemRects;
var
  TileHeight, LargeHeight, SmallHeight, WgHeight: Integer;

  //     
  procedure CalcIconsHeight;
  begin
    SmallHeight := -1;
    if (Images <> nil) and (ImageIndex >= 0) then
      SmallHeight := Images.Height;
    LargeHeight := -1;
    if (LargeImageList <> nil) and (LargeImageIndex >= 0) then
      LargeHeight := LargeImageList.Height;
    TileHeight := -1;
    if (TileImageList <> nil) and (TileImageIndex >= 0) then
      TileHeight := TileImageList.Height;
  end;

  //       
  //   -1,      
  function GetPreviosHeight(CurrentHeight: Integer): Integer;
  begin
    Result := -1;
    FUsedImageListIndex := -1;
    if (TileHeight < 0) and (LargeHeight < 0) and (SmallHeight < 0) then Exit;
    if CurrentHeight = -1 then
    begin
      if TileHeight > 0 then
      begin
        FUsedImageListIndex := 2;
        Result := TileHeight;
        Exit;
      end;
      if LargeHeight > 0 then
      begin
        FUsedImageListIndex := 1;
        Result := LargeHeight;
        Exit;
      end;
      if SmallHeight > 0 then
      begin
        FUsedImageListIndex := 0;
        Result := SmallHeight;
      end;
      Exit;
    end;
    if CurrentHeight = TileHeight then
    begin
      if LargeHeight > 0 then
      begin
        FUsedImageListIndex := 1;
        Result := LargeHeight;
        Exit;
      end;
      if SmallHeight > 0 then
      begin
        FUsedImageListIndex := 0;
        Result := SmallHeight;
      end;
      Exit;
    end;
    if CurrentHeight = LargeHeight then
      if SmallHeight > 0 then
      begin
        FUsedImageListIndex := 0;
        Result := SmallHeight;
      end;
  end;

  //    
  procedure CalcIconRect;
  const
    IconOffset = 2;
    IconDoubleOffset = IconOffset shl 1;
  var
    IcoHeight: Integer;
    CurrentHeight: Integer;
  begin
    ZeroMemory(@FIcoRect, SizeOf(TRect));

    //     
    IcoHeight := GetPreviosHeight(-1);
    if IcoHeight < 0 then Exit;

    //       ,  
    if IconPosition = ipTop then
      CurrentHeight := ClientHeight - IconDoubleOffset - WgHeight shl 1
    else
      CurrentHeight := ClientHeight - IconDoubleOffset;
    while CurrentHeight < IcoHeight do
    begin
      IcoHeight := GetPreviosHeight(IcoHeight);
      if IcoHeight < 0 then Exit;
    end;

    //        
    if IconPosition = ipLeft then
    begin
      FIcoRect.Left := IconDoubleOffset;
      FIcoRect.Top := (ClientHeight - IcoHeight) div 2;
    end
    else
    begin
      FIcoRect.Left := (ClientWidth - IcoHeight) div 2;
      FIcoRect.Top := IconDoubleOffset;
    end;
    FIcoRect.Right := FIcoRect.Left + IcoHeight;
    FIcoRect.Bottom := FIcoRect.Top + IcoHeight;
  end;

  //    
  procedure CalcMarkOffset;
  begin
    FDropDownLeftOffset := ClientWidth;
    //     
    //         
    if FShowCorner then
      if IconPosition = ipLeft then
        Dec(FDropDownLeftOffset, 8);
    if not FShowDropDownMark then Exit;
    Dec(FDropDownLeftOffset,  18);
  end;

  //    
  procedure CalcCaptionRect(ChekDescription: Boolean = True);
  begin
    ZeroMemory(@FCaptionRect, SizeOf(TRect));
    FCaptionSingleLine := True;
    if IconPosition = ipLeft then
    begin
      FCaptionRect.Left := FIcoRect.Right + 12;
      FCaptionRect.Right := FDropDownLeftOffset - 8;
      //   ,      
      if (Description = '') or not ChekDescription then
        FCaptionRect.Top := (ClientHeight - WgHeight) div 2
      else
        FCaptionRect.Top := 4;
      FCaptionRect.Bottom := FCaptionRect.Top + WgHeight;
    end
    else
    begin
      FCaptionRect.Top := FIcoRect.Bottom + 4;
      FCaptionRect.Bottom := ClientHeight - 4;
      FCaptionRect.Left := 8;
      FCaptionRect.Right := FDropDownLeftOffset - 4;
      if ShowDropDownMark then
        Inc(FCaptionRect.Right, 8);
      FCaptionSingleLine :=
        FCaptionRect.Bottom - FCaptionRect.Top < WgHeight shl 1;
    end;
  end;

  procedure CalcDescriptionRect;
  begin
    ZeroMemory(@FDescriptionRect, SizeOf(TRect));
    if Description = '' then Exit;
    if IconPosition = ipTop then Exit;
    FDescriptionRect.Top := FCaptionRect.Bottom + 2;
    FDescriptionRect.Bottom := ClientHeight - 2;
    if FDescriptionRect.Bottom - FDescriptionRect.Top < WgHeight then
    begin
      ZeroMemory(@FDescriptionRect, SizeOf(TRect));
      CalcCaptionRect(False);
      Exit;
    end;
    Dec(FDescriptionRect.Bottom, (FDescriptionRect.Bottom - FDescriptionRect.Top) mod WgHeight);
    FDescriptionSingleLine :=
      FDescriptionRect.Bottom - FDescriptionRect.Top < WgHeight shl 1;
    FDescriptionRect.Left := FIcoRect.Right + 12;
    FDescriptionRect.Right := FDropDownLeftOffset - 8;
  end;

begin
  //       -       
  if FShowPin then Exit;
  if Parent = nil then Exit;  
  WgHeight := Canvas.TextHeight('Wg');
  CalcIconsHeight;
  CalcIconRect;
  CalcMarkOffset;
  CalcCaptionRect;
  CalcDescriptionRect;
end;

procedure TFWBackStageButton.CMFontChanged(var Message: TMessage);
begin
  Invalidate;
end;

procedure TFWBackStageButton.CMMouseEnter(var Message: TMessage);
begin
  if FLocked then Exit;
  FButtonState := DXBAR_HOT;
  Invalidate;
end;

procedure TFWBackStageButton.CMMouseLeave(var Message: TMessage);
begin
  if FLocked then Exit;
  FButtonState := DXBAR_NORMAL;
  Invalidate;
end;

procedure TFWBackStageButton.CMTextChanged(var Message: TMessage);
begin
  if Caption <> '' then
    Invalidate;
end;

procedure TFWBackStageButton.CMThemeChanged(var Message: TMessage);
begin
  Invalidate;
end;

constructor TFWBackStageButton.Create(AOwner: TComponent);
begin
  inherited;
  FIconPosition := ipLeft;
  FLargeChangeLink := TChangeLink.Create;
  FLargeChangeLink.OnChange := ImageListChange;
  FSmallChangeLink := TChangeLink.Create;
  FSmallChangeLink.OnChange := ImageListChange;
  FTileChangeLink := TChangeLink.Create;
  FTileChangeLink.OnChange := ImageListChange;
  FLargeImageIndex := -1;
  FSmallImageIndex := -1;
  FTileImageIndex := -1;
  FBackGround := TBitmap.Create;
  FBackGround.PixelFormat := pf32bit;
  Width := 75;
  Height := 25;
  FButtonState := DXBAR_NORMAL;
  ControlStyle := [csCaptureMouse, csClickEvents, csOpaque];
  FCaptionColor := $262626;
  FDescriptionColor := clGrayText;
  FShowBorder := True;
  if ThemeServices.ThemesEnabled then
    ControlStyle := ControlStyle - [csOpaque]
  else
    ControlStyle := ControlStyle + [csOpaque];
  CalcItemRects;
end;

destructor TFWBackStageButton.Destroy;
begin
  FBackGround.Free;
  FSmallChangeLink.Free;
  FLargeChangeLink.Free;
  FTileChangeLink.Free;
  inherited;
end;

function TFWBackStageButton.GetActionLinkClass: TControlActionLinkClass;
begin
  Result := TFWBackStageButtonActionLink;
end;

function TFWBackStageButton.GetTransparent: Boolean;
begin
  Result := not (csOpaque in ControlStyle);
end;

procedure TFWBackStageButton.ImageListChange(Sender: TObject);
begin
  CalcItemRects;
  Invalidate;
end;

function TFWBackStageButton.IsAutoCheckStored: Boolean;
begin
  Result := not ((Action <> nil) and
    (TCustomAction(Action).AutoCheck = AutoCheck));
end;

function TFWBackStageButton.IsDownStored: Boolean;
begin
  Result := (ActionLink = nil) or not TFWBackStageButtonActionLink(ActionLink).IsCheckedLinked;
end;

function TFWBackStageButton.IsImageIndexStored(const Index: Integer): Boolean;
begin
  Result := (ActionLink = nil) or
    not TFWBackStageButtonActionLink(ActionLink).IsImageIndexLinked;
end;

function TFWBackStageButton.IsParentVisible: Boolean;
var
  P: TControl;
begin
  Result := False;
  P := Parent;
  if P = nil then Exit;  
  while P <> nil do
  begin
    if not P.Visible then Exit;
    P := P.Parent;
  end;
  Result := True;
end;

procedure TFWBackStageButton.Lock;
begin
  FLocked := True;
  FButtonState := DXBAR_PRESSED;
  Invalidate;
end;

procedure TFWBackStageButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
    if AComponent = Ribbon then
      Ribbon := nil;
end;

procedure TFWBackStageButton.Paint;

  procedure DrawDisabledBackGround;
  const
    ColorMap: array [0..1] of TColor = ($C9C8C8, $DEDDDD);
  var
    R{, SubRect}: TRect;
    CornerColor: array [0..3] of TColor;
  begin
    if not ShowBorder then Exit;
    R := GetClientRect;
    CornerColor[0] := FBackGround.Canvas.Pixels[0, 0];
    CornerColor[1] := FBackGround.Canvas.Pixels[R.Right - 1, 0];
    CornerColor[2] := FBackGround.Canvas.Pixels[R.Right - 1, R.Bottom - 1];
    CornerColor[3] := FBackGround.Canvas.Pixels[0, R.Bottom - 1];
    FBackGround.Canvas.Brush.Color := clWhite;
    FBackGround.Canvas.Pen.Color := ColorMap[0];
    FBackGround.Canvas.Rectangle(R);
    //   
    FBackGround.Canvas.Pixels[0, 0] := CornerColor[0];
    FBackGround.Canvas.Pixels[1, 0] := ColorMap[1];
    FBackGround.Canvas.Pixels[0, 1] := ColorMap[1];
    FBackGround.Canvas.Pixels[1, 1] := ColorMap[0];

    //   
    FBackGround.Canvas.Pixels[R.Right - 2, 0] := ColorMap[1];
    FBackGround.Canvas.Pixels[R.Right - 1, 0] := CornerColor[1];
    FBackGround.Canvas.Pixels[R.Right - 1, 1] := ColorMap[1];
    FBackGround.Canvas.Pixels[R.Right - 2, 1] := ColorMap[0];

    //   
    FBackGround.Canvas.Pixels[0, R.Bottom - 2] := ColorMap[1];
    FBackGround.Canvas.Pixels[0, R.Bottom - 1] := CornerColor[3];
    FBackGround.Canvas.Pixels[1, R.Bottom - 1] := ColorMap[1];
    FBackGround.Canvas.Pixels[1, R.Bottom - 2] := ColorMap[0];

    //   
    FBackGround.Canvas.Pixels[R.Right - 2, R.Bottom - 2] := ColorMap[0];
    FBackGround.Canvas.Pixels[R.Right - 1, R.Bottom - 2] := ColorMap[1];
    FBackGround.Canvas.Pixels[R.Right - 2, R.Bottom - 1] := ColorMap[1];
    FBackGround.Canvas.Pixels[R.Right - 1, R.Bottom - 1] := CornerColor[2];
  end;

  procedure DrawDefaultBackGround;
  const
    ColorMap: array [0..9] of TColor =
      ($EBEBEA, $E0DFDE, $ADABA9, $C6C6C5, $A8A6A5, $BFBDBC, $A4A2A1, $E2E2E1, $FFFFFF, $F1EFED);
  var
    R, SubRect: TRect;
    DC: HDC;
  begin
    if not Enabled then
    begin
      DrawDisabledBackGround;
      Exit;
    end;
    R := GetClientRect;
    DC := FBackGround.Canvas.Handle;
    //    
    SubRect := R;
    Inc(SubRect.Top, 2);
    Dec(SubRect.Bottom, 4);
    dxGpFillRectByGradient(DC, SubRect, ColorMap[1], ColorMap[2], LinearGradientModeVertical);
    //  
    SubRect := R;
    InflateRect(SubRect, -1, 0);
    FBackGround.Canvas.Brush.Color := clWhite;
    FBackGround.Canvas.FillRect(SubRect);
    SubRect.Top := SubRect.Bottom div 2;
    dxGpFillRectByGradient(DC, SubRect, ColorMap[8], ColorMap[9], LinearGradientModeVertical);
    //   
    FBackGround.Canvas.Pixels[1, 0] := ColorMap[0];
    FBackGround.Canvas.Pixels[0, 1] := ColorMap[0];
    FBackGround.Canvas.Pixels[1, 1] := ColorMap[1];
    //   
    FBackGround.Canvas.Pixels[R.Right - 2, 0] := ColorMap[0];
    FBackGround.Canvas.Pixels[R.Right - 1, 1] := ColorMap[0];
    FBackGround.Canvas.Pixels[R.Right - 2, 1] := ColorMap[1];
    //  
    FBackGround.Canvas.Pen.Color := ColorMap[1];
    FBackGround.Canvas.MoveTo(2, 0);
    FBackGround.Canvas.LineTo(R.Right - 2, 0);
    //   
    FBackGround.Canvas.Pixels[0, R.Bottom - 4] := ColorMap[2];
    FBackGround.Canvas.Pixels[0, R.Bottom - 3] := ColorMap[3];
    FBackGround.Canvas.Pixels[1, R.Bottom - 3] := ColorMap[4];
    FBackGround.Canvas.Pixels[1, R.Bottom - 2] := ColorMap[5];
    //   
    FBackGround.Canvas.Pixels[R.Right - 1, R.Bottom - 4] := ColorMap[2];
    FBackGround.Canvas.Pixels[R.Right - 1, R.Bottom - 3] := ColorMap[3];
    FBackGround.Canvas.Pixels[R.Right - 2, R.Bottom - 3] := ColorMap[4];
    FBackGround.Canvas.Pixels[R.Right - 2, R.Bottom - 2] := ColorMap[5];
    //  
    FBackGround.Canvas.Pen.Color := ColorMap[6];
    FBackGround.Canvas.MoveTo(2, R.Bottom - 2);
    FBackGround.Canvas.LineTo(R.Right - 2, R.Bottom - 2);
    // 
    FBackGround.Canvas.Pen.Color := ColorMap[7];
    FBackGround.Canvas.MoveTo(2, R.Bottom - 1);
    FBackGround.Canvas.LineTo(R.Right - 2, R.Bottom - 1);
  end;

  procedure DrawPin;
  var
    Painter: TdxRibbonBarPainter;
    R: TRect;
  begin
    Painter := TdxRibbonBarPainter.Create(Integer(Ribbon));
    try
      R := ClientRect;
      case FButtonState of
        DXBAR_NORMAL:
        begin
          if ShowBorder then
          begin
            DrawDefaultBackGround;
            InflateRect(R, -1, -1);
          end;
          if Down then
            Painter.Skin.DrawBackground(FBackGround.Canvas.Handle, R,
              DXBAR_MENUEXTRAPANE_PINBUTTON, DXBAR_CHECKED)
          else
            Painter.Skin.DrawBackground(FBackGround.Canvas.Handle, R,
              DXBAR_MENUEXTRAPANE_PINBUTTON, DXBAR_NORMAL);
        end;
        DXBAR_HOT,
        DXBAR_PRESSED:
        begin
          if Down then
            Painter.Skin.DrawBackground(FBackGround.Canvas.Handle, R,
              DXBAR_MENUEXTRAPANE_PINBUTTON, DXBAR_HOTCHECK)
          else
            Painter.Skin.DrawBackground(FBackGround.Canvas.Handle, R,
              DXBAR_MENUEXTRAPANE_PINBUTTON, DXBAR_HOT);
        end;
      end;
    finally
      Painter.Free;
    end;
  end;

  procedure DrawBackGround;
  var
    Painter: TdxRibbonBarPainter;
  begin
    Painter := TdxRibbonBarPainter.Create(Integer(Ribbon));
    try
      if not Enabled then
      begin
        DrawDisabledBackGround;
        Exit;
      end;
      case FButtonState of
        DXBAR_NORMAL:
          if Down then
            Painter.Skin.DrawBackground(FBackGround.Canvas.Handle, ClientRect,
              DXBAR_MENUITEM, DXBAR_CHECKED)
          else
            if ShowBorder then
              DrawDefaultBackGround
            else
             Painter.Skin.DrawBackground(FBackGround.Canvas.Handle, ClientRect,
               DXBAR_MENUITEM, FButtonState);
        DXBAR_HOT:
          if Down then
            Painter.Skin.DrawBackground(FBackGround.Canvas.Handle, ClientRect,
              DXBAR_MENUITEM, DXBAR_CHECKED)
          else
            Painter.Skin.DrawBackground(FBackGround.Canvas.Handle, ClientRect,
              DXBAR_MENUITEM, FButtonState);
        DXBAR_PRESSED:
          Painter.Skin.DrawBackground(FBackGround.Canvas.Handle, ClientRect,
            DXBAR_MENUITEM, FButtonState);
      end;
    finally
      Painter.Free;
    end;
  end;

  procedure ReverseGDIObject(ADC: HDC; APrevGDIObject: HGDIOBJ);
  begin
    DeleteObject(SelectObject(ADC, APrevGDIObject));
  end;

  procedure PolygonByColor(ADC: HDC; var APoints: array of TPoint; APenColor, ABrushColor: COLORREF);
  var
    APrevBrush, APrevPen: HGDIOBJ;
  begin
    APrevPen := SelectObject(ADC, CreatePen(PS_SOLID, 1, APenColor));
    APrevBrush := SelectObject(ADC, CreateSolidBrush(ABrushColor));
    Polygon(ADC, APoints, Length(APoints));
    ReverseGDIObject(ADC, APrevBrush);
    ReverseGDIObject(ADC, APrevPen);
  end;

  procedure DrawCorner;

    function Dark(Col: TColor; Percentage: Byte): TColor;
    var
      R, G, B: Byte;
    begin
      Col := ColorToRGB(Col);
      R := GetRValue(Col); G := GetGValue(Col); B := GetBValue(Col);
      R := Trunc(R * Percentage / 100);
      G := Trunc(G * Percentage / 100);
      B := Trunc(B * Percentage / 100);
      Result := RGB(R, G, B);
    end;

  var
    BorderColor: TColor;
    P: array[1..3] of TPoint;
    X, Y: Integer;
  begin
    if (FButtonState = DXBAR_NORMAL) and not Down then Exit;
    if Ribbon = nil then Exit;
    X := FBackGround.Width;
    Y := FBackGround.Height div 2;
    BorderColor := Dark(FBackGround.Canvas.Pixels[X - 1, Y], 75);
    P[1] := Point(X, Y - 6);
    P[2] := Point(X - 6, Y);
    P[3] := Point(X, Y + 6);
    PolygonByColor(FBackGround.Canvas.Handle, P, COLORREF(BorderColor), COLORREF(ColorToRGB(clWhite)));
  end;

  procedure DrawMark;
  const
    MarkSizeArrowY = 4;
    MarkSizeArrowX = 2 * MarkSizeArrowY - 1;
  var
    P: array[1..3] of TPoint;
    MarkR: TRect;
  begin
    MarkR := ClientRect;
    if IconPosition = ipTop then
      MarkR.Top := MarkR.Bottom - 22;
    MarkR.Left := FDropDownLeftOffset;
    MarkR.Right := MarkR.Left + 16;
    with MarkR, P[1] do
    begin
      P[1] := Point((Left + Right - MarkSizeArrowX) div 2,
        (Top + Bottom - MarkSizeArrowY) div 2 + (Top + Bottom - MarkSizeArrowY) mod 2);
      P[2] := Point(X + MarkSizeArrowX - 1, Y);
      P[3] := Point(X + MarkSizeArrowX div 2, Y + MarkSizeArrowY - 1);
    end;
    PolygonByColor(FBackGround.Canvas.Handle, P, FCaptionColor, FCaptionColor);
  end;

  procedure DrawIcoAndCaption;
  const
    AlignFlag: array[TAlignment] of Integer = (0, DT_RIGHT, DT_CENTER);
  var
    CurrentImageIndex: Integer;
    IL: TCustomImageList;
  begin

    case FUsedImageListIndex of
      0:
      begin
        IL := Images;
        CurrentImageIndex := ImageIndex;
      end;
      1:
      begin
        IL := FLargeImages;
        CurrentImageIndex := LargeImageIndex;
      end;
      2:
      begin
        IL := FTileImages;
        CurrentImageIndex := TileImageIndex;
      end
    else
      IL := nil;
      CurrentImageIndex := -1;
    end;

    if Assigned(IL) then
      IL.Draw(FBackGround.Canvas, FIcoRect.Left, FIcoRect.Top,
        CurrentImageIndex, Enabled);

    if ShowDropDownMark then
      DrawMark;

    FBackGround.Canvas.Brush.Style := bsClear;

    if Enabled then
      FBackGround.Canvas.Font.Color := FCaptionColor
    else
      FBackGround.Canvas.Font.Color := $B1B1B1;

    if (Description = '') or (IconPosition = ipTop) then
    begin
      if FCaptionSingleLine then
        DrawText(FBackGround.Canvas.Handle, PChar(Caption),
          Length(Caption), FCaptionRect, DT_WORD_ELLIPSIS or AlignFlag[Alignment])
      else
        DrawText(FBackGround.Canvas.Handle, PChar(Caption),
          Length(Caption), FCaptionRect, DT_WORD_ELLIPSIS or DT_WORDBREAK or AlignFlag[Alignment]);
    end
    else
      DrawText(FBackGround.Canvas.Handle, PChar(Caption),
        Length(Caption), FCaptionRect, DT_WORD_ELLIPSIS or AlignFlag[Alignment]);

    if Enabled then
      FBackGround.Canvas.Font.Color := FDescriptionColor
    else
      FBackGround.Canvas.Font.Color := $B1B1B1;

    if (Description <> '') and (FDescriptionRect.Bottom <> 0) then
    begin
      if FDescriptionSingleLine then
        DrawText(FBackGround.Canvas.Handle, PChar(Description), Length(Description),
          FDescriptionRect, DT_WORD_ELLIPSIS or AlignFlag[Alignment])
      else
        DrawText(FBackGround.Canvas.Handle, PChar(Description), Length(Description),
          FDescriptionRect, DT_WORD_ELLIPSIS or DT_WORDBREAK or AlignFlag[Alignment]);
    end;
    FBackGround.Canvas.Brush.Style := bsSolid;
  end;

begin
  inherited;
  if not ShowBorder and not (csOpaque in ControlStyle) then
    BitBlt(FBackGround.Canvas.Handle, 0, 0, Width, Height, Canvas.Handle, 0, 0, SRCCOPY)
  else
  begin
    FBackGround.Canvas.Brush.Color := Color;
    FBackGround.Canvas.FillRect(GetClientRect);
  end;
  if (csDesigning in ComponentState) then
  begin
    if (Ribbon = nil) or ((Ribbon <> nil) and not Down and not ShowBorder) then
    begin
      FBackGround.Canvas.Pen.Color := clGray;
      FBackGround.Canvas.Pen.Style := psDot;
      FBackGround.Canvas.Brush.Style := bsClear;
      FBackGround.Canvas.Rectangle(0, 0, ClientWidth, ClientHeight);
      FBackGround.Canvas.Brush.Style := bsSolid;
      FBackGround.Canvas.Pen.Style := psSolid;
    end;
  end;
  if Ribbon <> nil then
  begin
    if ShowPin then
      DrawPin
    else
    begin
      DrawBackGround;
      if ShowCorner then
        if IconPosition = ipLeft then
          DrawCorner;
    end;
  end
  else
    if ShowBorder then
      DrawDefaultBackGround;
  if not ShowPin then
    DrawIcoAndCaption;
  BitBlt(Canvas.Handle, 0, 0, Width, Height, FBackGround.Canvas.Handle, 0, 0, SRCCOPY);
end;

procedure TFWBackStageButton.SetAlignment(const Value: TAlignment);
begin
  FAlignment := Value;
  Invalidate;
end;

procedure TFWBackStageButton.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited;
  FBackGround.Width := AWidth;
  FBackGround.Height := AHeight;
  CalcItemRects;
end;

procedure TFWBackStageButton.SetCaptionColor(const Value: TColor);
begin
  FCaptionColor := Value;
  Invalidate;
end;

procedure TFWBackStageButton.SetDescriptionColor(const Value: TColor);
begin
  FDescriptionColor := Value;
  Invalidate;
end;

procedure TFWBackStageButton.SetDown(const Value: Boolean);
var
  P: TPoint;
begin
  FDown := Value;
  if Value then
    FButtonState := DXBAR_NORMAL //     Down    
  else
  begin
    FButtonState := DXBAR_HOT;
    GetCursorPos(P);
    if not PtInRect(ClientRect, ScreenToClient(P)) then
      FButtonState := DXBAR_NORMAL;
  end;
  Invalidate;
end;

procedure TFWBackStageButton.SetIconPosition(const Value: TIconPosition);
begin
  FIconPosition := Value;
  CalcItemRects;
  Invalidate;
end;

procedure TFWBackStageButton.SetImageIndex(const Index, Value: Integer);
begin
  case Index of
    0: FSmallImageIndex := Value;
    1: FLargeImageIndex := Value;
    2: FTileImageIndex := Value;
  end;
  CalcItemRects;
  Invalidate;
end;

procedure TFWBackStageButton.SetImages(const Index: Integer; const Value: TCustomImageList);

  procedure CheckImgListSize(List: TCustomImageList; CanBeSmaller: Boolean);
  const
    Err = '  TCustomImageList.   .';
  begin
    if (List = nil) or (Value = nil) then Exit;
    if CanBeSmaller then
    begin
      if List.Width >= Value.Width then
        raise EBackStageButtonException.Create(Err);
      if List.Height >= Value.Height then
        raise EBackStageButtonException.Create(Err);
    end
    else
    begin
      if List.Width <= Value.Width then
        raise EBackStageButtonException.Create(Err);
      if List.Height <= Value.Height then
        raise EBackStageButtonException.Create(Err);
    end;
  end;

var
  OwnImageList: ^TCustomImageList;
  OwnLink: TChangeLink;
begin
  case Index of
    0:
    begin
      CheckImgListSize(FLargeImages, False);
      CheckImgListSize(FTileImages, False);
      OwnImageList := @FSmallImages;
      OwnLink := FSmallChangeLink;
    end;
    1:
    begin
      CheckImgListSize(FSmallImages, True);
      CheckImgListSize(FTileImages, False);
      OwnImageList := @FLargeImages;
      OwnLink := FLargeChangeLink;
    end;
    2:
    begin
      CheckImgListSize(FSmallImages, True);
      CheckImgListSize(FLargeImages, True);
      OwnImageList := @FTileImages;
      OwnLink := FTileChangeLink;
    end;
  else
    OwnImageList := nil;
    OwnLink := nil;
  end;
  if OwnImageList = nil then Exit;  
  if OwnImageList^ <> Value then
  begin
    if OwnImageList^ <> nil then
      OwnImageList^.UnRegisterChanges(OwnLink);
    OwnImageList^ := Value;
    if OwnImageList^ <> nil then
    begin
      OwnImageList^.RegisterChanges(OwnLink);
      OwnImageList^.FreeNotification(Self);
    end;
    CalcItemRects;
    Invalidate;
  end
end;

procedure TFWBackStageButton.SetRibbon(const Value: TdxRibbon);
begin
  FRibbon := Value;
  Invalidate;
end;

procedure TFWBackStageButton.SetShowBorder(const Value: Boolean);
begin
  FShowBorder := Value;
  Invalidate;
end;

procedure TFWBackStageButton.SetShowCorner(const Value: Boolean);
begin
  FShowCorner := Value;
  CalcItemRects;
  Invalidate;
end;

procedure TFWBackStageButton.SetShowDropDownMark(const Value: Boolean);
begin
  FShowDropDownMark := Value;
  CalcItemRects;
  Invalidate;
end;

procedure TFWBackStageButton.SetShowPin(const Value: Boolean);
begin
  FShowPin := Value;
  CalcItemRects;
  Invalidate;
end;

procedure TFWBackStageButton.SetSubStrings(const Value: string);
begin
  FSubStrings := Value;
  CalcItemRects;
  Invalidate;
end;

procedure TFWBackStageButton.SetTransparent(const Value: Boolean);
begin
  if Transparent <> Value then
  begin
    if Value then
      ControlStyle := ControlStyle - [csOpaque]
    else
      ControlStyle := ControlStyle + [csOpaque];
    Invalidate;
  end;
  FTransparentSet := True;
end;

procedure TFWBackStageButton.Unlock;
var
  P: TPoint;
begin
  FLocked := False;
  FButtonState := DXBAR_HOT;
  GetCursorPos(P);
  if not PtInRect(ClientRect, ScreenToClient(P)) then
    FButtonState := DXBAR_NORMAL;
  Invalidate;
end;

procedure TFWBackStageButton.WMLButtonDown(var Message: TWMLButtonDown);
begin
  if FButtonState = DXBAR_PRESSED then Exit;
  if FLocked then Exit;
  if AutoCheck then
    Down := not Down;
  // OnDropDown   LBUTTONDOWN
  //     
  if ShowDropDownMark then
  begin
    if Assigned(FDropDown) then
      FDropDown(Self, Left, Top + Height, Width);
    CalcButtonStateAftefEvent;
    Exit;
  end;
  FButtonState := DXBAR_PRESSED;
  Invalidate;
end;

procedure TFWBackStageButton.WMLButtonUp(var Message: TWMLButtonUp);
begin
  if FLocked then Exit;
  Click;
  CalcButtonStateAftefEvent;
end;

{ TFWBackStageButtonActionLink }

procedure TFWBackStageButtonActionLink.AssignClient(AClient: TObject);
begin
  inherited AssignClient(AClient);
  FClient := AClient as TFWBackStageButton;
end;

function TFWBackStageButtonActionLink.IsCheckedLinked: Boolean;
begin
  Result := inherited IsCheckedLinked and
    (FClient.Down = (Action as TCustomAction).Checked);
end;

function TFWBackStageButtonActionLink.IsImageIndexLinked: Boolean;
begin
  Result := inherited IsImageIndexLinked and
    (FClient.ImageIndex = (Action as TCustomAction).ImageIndex);
end;

procedure TFWBackStageButtonActionLink.SetChecked(Value: Boolean);
begin
  FClient.Down := Value;

  {
   ,  Down  Click ,     

  if IsCheckedLinked then
  begin
    FClient.ClicksDisabled := True;
    try
      FClient.Checked := Value;
    finally
      FClient.ClicksDisabled := False;
    end;
  end; }
end;

procedure TFWBackStageButtonActionLink.SetImageIndex(Value: Integer);
begin
  if IsImageIndexLinked then
    FClient.ImageIndex := Value;
end;

end.
